package dev.shadowsoffire.apotheosis.util;

import net.minecraft.resources.ResourceLocation;
import net.minecraft.util.GsonHelper;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import net.minecraft.network.FriendlyByteBuf;
import com.google.gson.JsonObject;
import com.google.gson.JsonElement;
import net.minecraft.world.item.crafting.Ingredient;
import net.minecraftforge.common.crafting.IIngredientSerializer;
import java.util.Iterator;
import java.util.Collection;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.Gem;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixHelper;
import java.util.stream.Stream;
import net.minecraft.world.item.ItemStack;
import dev.shadowsoffire.apotheosis.adventure.loot.LootRarity;
import dev.shadowsoffire.placebo.reload.DynamicHolder;
import net.minecraftforge.common.crafting.AbstractIngredient;

public class GemIngredient extends AbstractIngredient
{
    protected final DynamicHolder<LootRarity> rarity;
    protected ItemStack[] items;
    
    public GemIngredient(final DynamicHolder<LootRarity> rarity) {
        super((Stream)Stream.empty());
        this.rarity = rarity;
    }
    
    public boolean test(final ItemStack stack) {
        final DynamicHolder<LootRarity> rarity = (DynamicHolder<LootRarity>)AffixHelper.getRarity(stack);
        return stack.m_41720_() == Adventure.Items.GEM.get() && rarity.isBound() && rarity == this.rarity;
    }
    
    public ItemStack[] m_43908_() {
        if (this.items == null) {
            final Collection<Gem> gems = GemRegistry.INSTANCE.getValues();
            if (gems.size() == 0) {
                return new ItemStack[0];
            }
            this.items = new ItemStack[gems.size()];
            int i = 0;
            for (final Gem g : GemRegistry.INSTANCE.getValues()) {
                this.items[i++] = GemRegistry.createGemStack(g, (LootRarity)this.rarity.get());
            }
        }
        return this.items;
    }
    
    protected void invalidate() {
        super.invalidate();
        this.items = null;
    }
    
    public boolean isSimple() {
        return false;
    }
    
    public IIngredientSerializer<? extends Ingredient> getSerializer() {
        return (IIngredientSerializer<? extends Ingredient>)Serializer.INSTANCE;
    }
    
    public JsonElement m_43942_() {
        return new JsonObject();
    }
    
    public LootRarity getRarity() {
        return (LootRarity)this.rarity.get();
    }
    
    public static class Serializer implements IIngredientSerializer<GemIngredient>
    {
        public static final Serializer INSTANCE;
        
        public GemIngredient parse(final FriendlyByteBuf buffer) {
            final DynamicHolder<LootRarity> rarity = (DynamicHolder<LootRarity>)RarityRegistry.INSTANCE.holder(buffer.m_130281_());
            return new GemIngredient(rarity);
        }
        
        public GemIngredient parse(final JsonObject json) {
            final DynamicHolder<LootRarity> rarity = (DynamicHolder<LootRarity>)RarityRegistry.INSTANCE.holder(new ResourceLocation(GsonHelper.m_13906_(json, "rarity")));
            return new GemIngredient(rarity);
        }
        
        public void write(final FriendlyByteBuf buffer, final GemIngredient ingredient) {
            buffer.m_130085_(ingredient.rarity.getId());
        }
        
        static {
            INSTANCE = new Serializer();
        }
    }
}
